#pragma once

#include <ftl/rgbd/detail/source.hpp>
#include <ftl/configuration.hpp>
#include <opencv2/core.hpp>

namespace ftl {
namespace rgbd {

class Frame {
public:
	/* @todo	TODO: REMOVE HARDCODED CHANNEL COUNT!
	 */
	Frame() : channels_host_(11), channels_gpu_(11), available_(11, 0) {}

	/* @brief	Reset all channels without releasing memory.
	 */
	void reset()
	{
		std::fill(available_.begin(), available_.end(), 0);
	}

	/* @brief	Is there valid data in channel (either host or gpu).
	 */
	bool hasChannel(const ftl::rgbd::channel_t& channel)
	{
		return available_[_channelIdx(channel)];
	}

	/* @brief	get reference to the channel contents
	 * @param	channel type
	 * @returns	const reference to channel data
	 * 
	 * Methods automatically copy between host/gpu if the data is only available
	 * in the other. Results are cached. Result is valid only if hasChannel() is
	 * true.
	 */

	template <typename T> const T& getChannel(const ftl::rgbd::channel_t& channel);
	template <typename T> T& setChannel(const ftl::rgbd::channel_t& channel);

private:

	static size_t _channelIdx(const ftl::rgbd::channel_t& channel)
	{
		switch(channel)
		{
			case kChanNone:				return 0;
			case kChanLeft:				return 1;
			case kChanDepth:			return 2;
			case kChanRight:			return 3;
			case kChanDisparity:		return 4;
			case kChanDeviation:		return 5;
			case kChanNormals:			return 6;
			case kChanConfidence:		return 7;
			case kChanFlow:				return 8;
			case kChanEnergy:			return 9;
			// should not happen (error); returned index is kChanNone
			default:					return 0;
		}
	}

	std::vector<cv::Mat> channels_host_;
	std::vector<cv::cuda::GpuMat> channels_gpu_;

	// bitmasks for each channel stored in available_
	static const uint mask_host = 1;
	static const uint mask_gpu = 2;

	std::vector<uint> available_;
};

}
}
