
#include <ftl/rgbd/frame.hpp>

namespace ftl {
namespace rgbd {

template<> const cv::Mat& Frame::getChannel(const ftl::rgbd::channel_t& channel)
{
	auto idx = _channelIdx(channel);
	if (!(available_[idx] & mask_host))
	{
		if (available_[idx] & mask_gpu)
		{
			channels_gpu_[idx].download(channels_host_[idx]);
			available_[idx] |= mask_host;
		}
	}

	return channels_host_[idx];
}

template<> cv::Mat& Frame::setChannel(const ftl::rgbd::channel_t& channel)
{
	auto idx = _channelIdx(channel);
	available_[idx] = mask_host;
	return channels_host_[idx];
}

template<> const cv::cuda::GpuMat& Frame::getChannel(const ftl::rgbd::channel_t& channel)
{
	auto idx = _channelIdx(channel);
	if (!(available_[idx] & mask_gpu))
	{
		if (available_[idx] & mask_host)
		{
			channels_gpu_[idx].upload(channels_host_[idx]);
			available_[idx] |= mask_gpu;
		}
	}
	
	return channels_gpu_[idx];
}

template<> cv::cuda::GpuMat& Frame::setChannel(const ftl::rgbd::channel_t& channel)
{
	auto idx = _channelIdx(channel);
	available_[idx] = mask_gpu;
	return channels_gpu_[idx];
}

}
}